from sys import prefix
from orjson import loads
from .route import Route
from .request import Request
from .response import Response


class WSGI:
	def __init__(self, default_content_type="application/json", status_codes={}):
		self._content_type = default_content_type
		if type(status_codes) == dict and type(status_codes.get("text/html")) == dict:
			self.codes = status_codes
		else:
			file = open(prefix + "/WeefPy/codes.json", "rb")
			self.codes = loads(file.read())

	def __call__(self, env, start_response):
		path = env["PATH_INFO"] if env["PATH_INFO"][-1] == "/" else env["PATH_INFO"] + "/"

		request = Request(env)
		response = Response(start_response, default_content_type=self._content_type)

		if request.get_bad_request():
			response.set(self.get_code(400), status="400 BAD REQUEST")
		else:
			method, args, method_not_allowed = Route.get(path, env["REQUEST_METHOD"])
			if not method:
				data = None
				if method_not_allowed:
					response.set(self.get_code(405), status="405 METHOD NOT ALLOWED")
				else:
					response.set(self.get_code(404), status="404 NOT FOUND")
			else:
				data = method(request, response, *args)

			if data:
				if type(data) == tuple:
					response.set(*data)
				else:
					response.set(data)
			elif not response.get_status():
				response.set(self.get_code(501), status="501 NOT FOUND")

		response()
		yield response.get_body()

	def get_code(self, code):
		codes = self.codes.get(self._content_type, self.codes["text/html"])
		response = codes.get(str(code))
		if not response:
			if self._content_type == "application/json":
				response = "{\"status\": \"\", \"code\": %d, \"message\": \"\"}" % code
			else:
				response = str(code)
		return response
